//! ## Tests
//! Some unitary tests grouped here <br>
//! [`test_expected_metrics`] the more important one to verify results with **output_expected.txt**
//!
use super::*;
#[test]
fn test_load_input() -> Result<(), String> {
    let metrics_target = MetricsConsumerTarget {
        input: String::from(DEFAULT_INPUT_PATH),
        reg_exp: String::from(DEFAULT_REG_EXP),
        ..Default::default()
    };
    match metrics_target.load_input() {
        Ok(_) => Ok(()),
        Err(e) => Err(format!("Error: {}", e).into()),
    }
}
#[test]
fn test_invalid_line_value() -> Result<(), String> {
    let metrics_target = MetricsConsumerTarget {
        input: String::from(DEFAULT_INPUT_PATH),
        reg_exp: String::from(DEFAULT_REG_EXP),
        ..Default::default()
    };
    let contents = String::from("1650973075 cpu A47\n");
    let re = regex::Regex::new(&metrics_target.reg_exp)
        .map_err(|err| format!("Error regex: {}", err))?;
    match metrics_target.parse_line(&contents, 1, &re) {
        Some(_) => Err(format!("Error invalid line value: {}", contents).into()),
        None => Ok(()),
    }
}
#[test]
fn test_invalid_line_time() -> Result<(), String> {
    let metrics_target = MetricsConsumerTarget {
        input: String::from(DEFAULT_INPUT_PATH),
        reg_exp: String::from(DEFAULT_REG_EXP),
        ..Default::default()
    };
    let contents = String::from("1650973075A cpu 47\n");
    let re = regex::Regex::new(&metrics_target.reg_exp)
        .map_err(|err| format!("Error regex: {}", err))?;
    match metrics_target.parse_line(&contents, 1, &re) {
        Some(_) => Err(format!("Error invalid line value: {}", contents).into()),
        None => Ok(()),
    }
}
#[test]
fn test_expected_metrics() {
    use std::{
        fs::File,
        io::{prelude::*, BufReader},
    };
    let metrics_target = MetricsConsumerTarget {
        input: String::from(DEFAULT_INPUT_PATH),
        output: String::from("vec"),
        reg_exp: String::from(DEFAULT_REG_EXP),
        ..Default::default()
    };
    let mut result = metrics_target.load_input().unwrap_or_default();
    result.collect_aggregates();
    let data_metrics = result.show_metrics().unwrap_or_default();

    let expected_output = String::from("output_expected.txt");
    let file = File::open(expected_output.clone())
        .expect(format!("no such file: {}", expected_output).as_str());
    let buf = BufReader::new(file);
    let lines: Vec<String> = buf
        .lines()
        .map(|l| l.expect("Could not parse line"))
        .collect();
    assert_eq!(lines.join("\n"), data_metrics.join("\n"));
}
